package com.factory.base.util.res;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.factory.base.entity.aggregates.PageBean;
import com.factory.base.entity.aggregates.ResponseBean;
import com.factory.base.entity.aggregates.ResponseEnum;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.factory.base.entity.aggregates.ResponseEnum.OK;


/**
 * 构建统一响应格式
 *
 * @param <T>
 */

public class ResponseBeanBuilder<T> {

    /**
     * 相应单一对象
     *
     * @param
     * @param
     * @return
     */
    public static <T> ResponseBean<T> ok() {
        return ResponseBean.<T>builder().status(OK.getStatus()).message(OK.getMessage()).build();
    }

    /**
     * 相应单一对象
     *
     * @param
     * @param
     * @return
     */
    public static <T> ResponseBean<T> ok(String status, String message) {
        return ResponseBean.<T>builder().status(status).message(message).build();
    }

    /**
     * 相应单一对象
     *
     * @param data
     * @param <T>
     * @return
     */
    public static <T> ResponseBean<T> ok(T data) {
        return ResponseBean.<T>builder().status(OK.getStatus()).message(OK.getMessage()).data(data).build();
    }

    /**
     * 相应单一对象
     *
     * @param data
     * @param <T>
     * @return
     */
    public static <T> ResponseBean<T> ok(T data, String message) {
        return ResponseBean.<T>builder().status(OK.getStatus()).message(message).data(data).build();
    }

    /**
     * 相应单一对象
     *
     * @param page
     * @param <T>
     * @return
     */
    public static <T> ResponseBean<PageBean<T>> ok(IPage<T> page) {
        return ResponseBean
                .<PageBean<T>>builder()
                .status(OK.getStatus())
                .message(OK.getMessage())
                .data(buildPageBean(page))
                .build();
    }


    /**
     * 构建分页
     *
     * @param function
     * @param <T>
     * @param <R>
     * @return
     */
    public static <T, R> ResponseBean<List<R>> ok(List<T> dataList, Function<? super T, ? extends R> function) {
        return ResponseBean.<List<R>>builder()
                .data(buildBean(dataList, function))
                .status(OK.getStatus())
                .message(OK.getMessage())
                .build();
    }

    /**
     * 构建分页
     *
     * @param page
     * @param function
     * @param <T>
     * @param <R>
     * @return
     */
    public static <T, R> ResponseBean<PageBean<R>> ok(IPage<T> page, Function<? super T, ? extends R> function) {
        return ResponseBean.<PageBean<R>>builder()
                .data(buildPageBean(page, function))
                .status(OK.getStatus())
                .message(OK.getMessage())
                .build();
    }


    /**
     * 构建分页
     *
     * @param function
     * @param <T>
     * @param <R>
     * @return
     */
    public static <T, R> List<R> buildBean(List<T> dataList, Function<? super T, ? extends R> function) {
        return dataList.stream().map(function).collect(Collectors.toList());
    }

    public static <T> PageBean<T> buildPageBean(IPage<T> page) {
        List<Integer> pageList = new ArrayList<>();
        for (int i = 1; i <= page.getPages(); i++) {
            pageList.add(i);
        }
        return PageBean
                .<T>builder()
                .row(
                        page.getRecords()
                )
                .total(page.getTotal())
                .page(page.getPages())
                .pages(pageList)
                .build();
    }

    /**
     * 构建分页
     *
     * @param page
     * @param function
     * @param <T>
     * @param <R>
     * @return
     */
    public static <T, R> PageBean<R> buildPageBean(IPage<T> page, Function<? super T, ? extends R> function) {
        List<Integer> pageList = new ArrayList<>();
        for (int i = 1; i <= page.getPages(); i++) {
            pageList.add(i);
        }
        return PageBean
                .<R>builder()
                .row(
                        page.getRecords().stream().map(function).collect(Collectors.toList())
                )
                .total(page.getTotal())
                .page(page.getPages())
                .pages(pageList)
                .build();
    }

    /**
     * 失败
     *
     * @return
     */
    public static <T> ResponseBean<T> fail() {
        return ResponseBean.<T>builder().status(ResponseEnum.FAIL.getStatus()).message(ResponseEnum.FAIL.getMessage()).build();
    }

    /**
     * 失败
     */
    public static <T> ResponseBean<T> fail(String status, String message) {
        return ResponseBean.<T>builder().status(status).message(message).build();
    }

    /**
     * 系统异常类并返回结果数据
     *
     * @param errorType
     * @param data
     * @return Result
     */
    public static <T> ResponseBean<T> fail(ResponseEnum errorType, T data) {
        return ResponseBean
                .<T>builder()
                .status(errorType.getStatus())
                .message(errorType.getMessage())
                .data(data).build();
    }

    /**
     * 失败
     */
    public static <T> ResponseBean<T> fail(String message) {
        return ResponseBean.<T>builder().status(ResponseEnum.FAIL.getStatus()).message(message).build();
    }

    /**
     * 失败
     *
     * @return
     */
    public static <T> ResponseBean<T> fail(T data) {
        return ResponseBean.<T>builder().status(ResponseEnum.FAIL.getStatus()).message(ResponseEnum.FAIL.getMessage()).data(data).build();
    }

    public static <T> ResponseBean<T> fail(ResponseEnum responseEnum) {
        return ResponseBean.<T>builder().status(responseEnum.getStatus()).message(responseEnum.getMessage()).build();
    }

}
